# Copyright 2017 Palantir Technologies, Inc.
import configparser
import logging
import os
import sys

log = logging.getLogger(__name__)


class ConfigSource(object):
    """Base class for implementing a config source."""

    def __init__(self, root_path):
        self.root_path = root_path
        self.is_windows = sys.platform == 'win32'
        self.xdg_home = os.environ.get(
            'XDG_CONFIG_HOME', os.path.expanduser('~/.config')
        )

    def user_config(self):
        """Return user-level (i.e. home directory) configuration."""
        raise NotImplementedError()

    def project_config(self, document_path):
        """Return project-level (i.e. workspace directory) configuration."""
        raise NotImplementedError()

    @staticmethod
    def read_config_from_files(files):
        config = configparser.RawConfigParser()
        for filename in files:
            if os.path.exists(filename) and not os.path.isdir(filename):
                config.read(filename)

        return config

    @staticmethod
    def parse_config(config, key, options):
        """Parse the config with the given options."""
        conf = {}
        for source, destination, opt_type in options:
            opt_value = _get_opt(config, key, source, opt_type)
            if opt_value is not None:
                _set_opt(conf, destination, opt_value)
        return conf


def _get_opt(config, key, option, opt_type):
    """Get an option from a configparser with the given type."""
    for opt_key in [option, option.replace('-', '_')]:
        if not config.has_option(key, opt_key):
            continue

        if opt_type == bool:
            return config.getboolean(key, opt_key)

        if opt_type == int:
            return config.getint(key, opt_key)

        if opt_type == str:
            return config.get(key, opt_key)

        if opt_type == list:
            return _parse_list_opt(config.get(key, opt_key))

        raise ValueError("Unknown option type: %s" % opt_type)


def _parse_list_opt(string):
    return [s.strip() for s in string.split(",") if s.strip()]


def _set_opt(config_dict, path, value):
    """Set the value in the dictionary at the given path if the value is not None."""
    if value is None:
        return

    if '.' not in path:
        config_dict[path] = value
        return

    key, rest = path.split(".", 1)
    if key not in config_dict:
        config_dict[key] = {}

    _set_opt(config_dict[key], rest, value)



# Copyright 2017 Palantir Technologies, Inc.
import logging
from pyls import hookimpl, lsp

log = logging.getLogger(__name__)


@hookimpl
def pyls_document_highlight(document, position):
    usages = document.jedi_script(position).usages()

    def is_valid(definition):
        return definition.line is not None and definition.column is not None

    def local_to_document(definition):
        return not definition.module_path or definition.module_path == document.path

    return [{
        'range': {
            'start': {'line': d.line - 1, 'character': d.column},
            'end': {'line': d.line - 1, 'character': d.column + len(d.name)}
        },
        'kind': lsp.DocumentHighlightKind.Write if d.is_definition() else lsp.DocumentHighlightKind.Read
    } for d in usages if is_valid(d) and local_to_document(d)]



# Copyright 2017 Palantir Technologies, Inc.
# pylint: disable=redefined-builtin, unused-argument
from pyls import hookspec


@hookspec
def pyls_code_actions(config, workspace, document, range, context):
    pass


@hookspec
def pyls_code_lens(config, workspace, document):
    pass


@hookspec
def pyls_commands(config, workspace):
    """The list of command strings supported by the server.

    Returns:
        List[str]: The supported commands.
    """


@hookspec
def pyls_completions(config, workspace, document, position):
    pass


@hookspec
def pyls_definitions(config, workspace, document, position):
    pass


@hookspec
def pyls_dispatchers(config, workspace):
    pass


@hookspec
def pyls_document_did_open(config, workspace, document):
    pass


@hookspec
def pyls_document_did_save(config, workspace, document):
    pass


@hookspec
def pyls_document_highlight(config, workspace, document, position):
    pass


@hookspec
def pyls_document_symbols(config, workspace, document):
    pass


@hookspec(firstresult=True)
def pyls_execute_command(config, workspace, command, arguments):
    pass


@hookspec
def pyls_experimental_capabilities(config, workspace):
    pass


@hookspec(firstresult=True)
def pyls_folding_range(config, workspace, document):
    pass


@hookspec(firstresult=True)
def pyls_format_document(config, workspace, document):
    pass


@hookspec(firstresult=True)
def pyls_format_range(config, workspace, document, range):
    pass


@hookspec(firstresult=True)
def pyls_hover(config, workspace, document, position):
    pass


@hookspec
def pyls_initialize(config, workspace):
    pass


@hookspec
def pyls_initialized():
    pass


@hookspec
def pyls_lint(config, workspace, document, is_saved):
    pass


@hookspec
def pyls_references(config, workspace, document, position, exclude_declaration):
    pass


@hookspec(firstresult=True)
def pyls_rename(config, workspace, document, position, new_name):
    pass


@hookspec
def pyls_settings(config):
    pass


@hookspec(firstresult=True)
def pyls_signature_help(config, workspace, document, position):
    pass
